/***************************************************************************
                          apollon.cpp -  description
                             -------------------
begin		: Dec 05 2003
copyright	: (C) 2002 by Bastiaan Naber, (C) 2002 by Robert Wittams
email		: bastiaan@ricardis.tudelft.nl
description	: 

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <iostream>
using namespace std;
#include <stdlib.h>
#include <signal.h>


#include <qstring.h>
#include <qvbox.h>
#include <qapplication.h>
#include <qstringlist.h>

#include <kstatusbar.h>
#include <kmenubar.h>
#include <kpopupmenu.h>
#include <kiconloader.h>
#include <kcmdlineargs.h>
#include <kconfig.h>
#include <klocale.h>
#include <kstdaccel.h>
#include <kstdaction.h>

#include "inifile.h"
#include "apollon.h"
#include "apolloninfotab.h"
#include "apollonsearchtab.h"
#include "apollontransfertab.h"
#include "apollonplayertab.h"
#include "apollonsharedviewtab.h"

ApollonApp::ApollonApp( QWidget *parent, const char *name ): KMainWindow( parent, name )
{
	int port;
	QString host;
	mStartDocked= FALSE;
	gcn = new giFTConnection();

	settingsDialog = new ApollonPreferencesDialog();
	mySigDialog = new SigDialog(gcn);

	rightstats=new QLabel( this ) ;
	rightstats ->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	rightstats ->setAlignment( AlignBottom | AlignRight );

	connect(gcn, SIGNAL( connected() ), this, SLOT( slotConnected() ));
	connect(gcn, SIGNAL( giftShareReply(QString) ), this, SLOT( giftShareReply(QString) ));

	connect(gcn, SIGNAL(sigGiFTPID(pid_t)), this, SLOT(sltGiFTPID(pid_t)));

	KCmdLineArgs *args = KCmdLineArgs::parsedArgs();

	QString commandlineserver = args->getOption("s");

	if (!commandlineserver)
	{
		/**************************** This could be set on the command line also ****************/
		QString user(getenv("USER"));
		QString home(getenv("HOME"));
		QString filename(home);
		filename.append("/.giFT/ui/ui.conf");

		IniFile* uiconf = new IniFile( filename );

		IniSection* daemon = uiconf->getSection( "daemon" );

		//cout << daemon->toString() << endl;

		port = daemon->value("port").toInt();

		host = daemon->value("host");

		//cout << host << " : " << port << endl;
		/**************************** This could be set on the command line also ****************/
	}
	else
	{
		// This is not idiot proof but for now it works!
		QStringList PortHostList = QStringList::split( QString( ":" ), commandlineserver);
		port = PortHostList[1].toInt();
		host = PortHostList[0];
	}


	if (args->isSet("d")) {   // apollon -d enables debug output
	  gcn->setDebugOutput(args->isSet("d")); //here you can add other objects which might have debug-output
	}

	gcn->setParams( host, port );
	gcn->connectToServer();
	pkill = true;
	stop = false;
	initStatusBar();
	initActions();
	initCentre();
	setupSystemTray();
	readConfig();

}

bool ApollonApp::queryClose()
{
	if(stop)
	{
		return true;
	}

	hide();
	return false;
}

void ApollonApp::quit()
{
	qApp->quit();
	delete this;
}

ApollonApp::~ApollonApp()
{
	if(pkill)
	{
	  // kill(giFT_pid, 9);
	  system("killall giftd");
	}
}

void ApollonApp::initStatusBar()
{
	statusBar()->message(i18n("Connecting..."));
	rightstats ->setText("                      ");
	statusBar()->addWidget ( rightstats,  0, TRUE );
}
void ApollonApp::setRightStatus(int a, float r)
{
        QString rstats;
	rstats = i18n("%1 active downloads at %2 KB/s.").arg(a).arg(r);
	rightstats ->setText( rstats);
}

void ApollonApp::readConfig() {

  setAutoSaveSettings();
  settingsDialog->readConfig();
  setAutoSaveSettings("MainWindow");
}

void ApollonApp::saveConfig() {//this slot gets called, when the settings are changed, therefore
  // we also have to re-send the informations
  settingsDialog->saveConfig();
}

void ApollonApp::initActions()
{
	KAction *fileExitAction =        KStdAction::quit(                              this,           SLOT(quit()),       actionCollection(), "fileExitAction");
	KAction *sharesHideAction =      new KAction(i18n("&Hide Shares"),           0, this,           SLOT(sharesHide()), actionCollection(), "sharesHideAction");
	KAction *sharesShowAction =      new KAction(i18n("&Show Shares"),           0, this,           SLOT(sharesShow()), actionCollection(), "sharesShowAction");
	KAction *sharesSyncAction =      new KAction(i18n("S&ync"),        "reload", 0, this,           SLOT(sharesSync()), actionCollection(), "sharesSyncAction");
	KAction *settingsOptionsAction = KStdAction::preferences(                       settingsDialog, SLOT(exec()),       actionCollection(), "settingsOptionsAction");
	KAction *sigAddlinkAction =      new KAction(i18n("&Add Sig2Dat link"), "bookmark_add", 0, mySigDialog, SLOT(exec()), actionCollection(), "sigAddlinkAction");

	// create File menu
	KPopupMenu *fileMenu = new KPopupMenu(this);
	sigAddlinkAction->plug(fileMenu);
	fileMenu->insertSeparator();
	fileExitAction->plug(fileMenu);

	// create Shares menu
	KPopupMenu *sharesMenu = new KPopupMenu(this);
	sharesHideAction->plug(sharesMenu);
	sharesShowAction->plug(sharesMenu);
	sharesMenu->insertSeparator();
	sharesSyncAction->plug(sharesMenu);

	// create Settings menu
	KPopupMenu *settingsMenu = new KPopupMenu(this);
	settingsOptionsAction->plug(settingsMenu);

	// assign Help menu
	KPopupMenu *helpMenu_ = helpMenu();

	// insert menus into the menubar
	menuBar()->insertItem(i18n("&File"),     fileMenu,     0,  ALT + Key_F);
	menuBar()->insertItem(i18n("&Shares"),   sharesMenu,   0,  ALT + Key_S);
	menuBar()->insertItem(i18n("S&ettings"), settingsMenu, 0,  ALT + Key_E);
	menuBar()->insertItem(i18n("&Help"),     helpMenu_,     0, ALT + Key_H);

	connect(settingsDialog, SIGNAL(sigOK()), this, SLOT(saveConfig()));
	connect(settingsDialog, SIGNAL(sigShutApollon(bool)), this, SLOT(sltSetShutApollon(bool)));
	connect(settingsDialog, SIGNAL(sigStartDocked(bool)), this, SLOT(sltStartDocked(bool)));
	connect(settingsDialog, SIGNAL(sigShowPopUps(bool)), this, SLOT(sltShowPopUps(bool)));
	connect(settingsDialog, SIGNAL(sigStartPage(int)), this, SLOT(sltStartPage(int)));
	connect(settingsDialog, SIGNAL(sigRestartDaemon()), gcn, SLOT(sltRestartDaemon()));
	connect(settingsDialog, SIGNAL(sigKillDaemon(bool)), this, SLOT(sltKillgiFT(bool)));

}

void ApollonApp::initCentre()
{
	KIconLoader *loader = KGlobal::iconLoader();

	tabw = new KJanusWidget(this, "tabw", KJanusWidget::IconList);

	setCentralWidget(tabw);

	infobox = tabw->addVBoxPage(i18n("Info"), NULL, loader->loadIcon("messagebox_info", KIcon::Small, 32) );

	searchbox = tabw->addVBoxPage(i18n("Search"), NULL, loader->loadIcon("find", KIcon::Small, 32));

	transbox = tabw->addVBoxPage(i18n("Transfers"), NULL, loader->loadIcon("network", KIcon::Small, 32) );

	sharedbox = tabw->addVBoxPage(i18n("Shared folders"), NULL, loader->loadIcon("folder", KIcon::Small,32));

	playerbox = tabw->addVBoxPage(i18n("Player"), NULL, loader->loadIcon("kmix", KIcon::Small, 32));

	info = new ApollonInfoTab(infobox, "ApollonInfoTab", gcn);
	search = new ApollonSearchTab(searchbox, "ApollonSearchTab", gcn);
	trans = new ApollonTransferTab(transbox, "ApollonTransferTab", gcn);
	sharedView = new ApollonSharedViewTab(sharedbox, "ApollonSharedViewTab");
	player = new ApollonPlayerTab(playerbox, "ApollonPlayerTab", tabw, 4);

	trans->setPreferences(settingsDialog);
	trans->setPlayerTab(player);
	sharedView->setPlayerTab(player);

	connect(settingsDialog, SIGNAL(sigAutoSearchInterval(int)), trans, SLOT(sltAutoSearchInterval(int)));
	connect(settingsDialog, SIGNAL(sigSharedFolder(QString)), sharedView, SLOT(addFolder(QString)));

	connect(info, SIGNAL(sigKill(bool)), this, SLOT(sltKillgiFT(bool)));
	connect(info, SIGNAL(statusMessage(QString)), this, SLOT(slotSetStatus(QString)) );
	connect(trans, SIGNAL(sigBrowseUser(QString)), search, SLOT(BrowseUser(QString)));
	connect(trans, SIGNAL(sigBrowseUser(QString)), this, SLOT(sltShowSearches(QString)));
	connect(trans, SIGNAL(sigSendInfo(int, float)), this, SLOT(setRightStatus(int, float)));
}


void ApollonApp::slotSetStatus( QString text )
{
	statusBar()->clear();
	statusBar()->message( text );
}

void ApollonApp::slotConnected()
{
	//statusBar()->message("Connected to giFT deamon");
}

void ApollonApp::sharesSync()
{
	gcn->makeShareRequest("sync");
}

void ApollonApp::sharesHide()
{
	gcn->makeShareRequest("hide");
}

void ApollonApp::sharesShow()
{
	gcn->makeShareRequest("show");
}

void ApollonApp::giftShareReply( QString reply)
{
	if (reply.find("status") > -1)
	{
		slotSetStatus(i18n("Synchronizing shares: ") + giFTConnection::searchKey("status", reply));

		if (reply.find("Done") > -1)
		{
			gcn->makeStatsRequest();
		}
	}
	else
	{
		if (reply.find("hide") > -1)
		{
			slotSetStatus(i18n("Shares are now hidden."));
			gcn->makeStatsRequest();
		}
		else
		{
			slotSetStatus(i18n("Shares are now shown."));
			gcn->makeStatsRequest();
		}
	}
}

void ApollonApp::sltShowSearches(QString)
{
	tabw->showPage(1);
}


void ApollonApp::setupSystemTray()
{
   	systemTray = new ApollonSystemTray(this, "systemTray");
	systemTray->show();
	trans->setTray(systemTray);
	connect(systemTray, SIGNAL(quitSelected()),          this,       SLOT(quit()));
	connect(trans,      SIGNAL(sigSendInfo(int, float)), systemTray, SLOT(sltInfo(int, float)));
}


void ApollonApp::sltGiFTPID(pid_t pid) {
  giFT_pid = pid;
}

void ApollonApp::sltKillgiFT(bool kil) {
  pkill = kil;
}


bool ApollonApp::startDocked() {
  return mStartDocked;
}

void ApollonApp::sltShowPopUps(bool show) {
  trans->setShowPopup(show);}

void ApollonApp::sltStartPage(int  i) { 
  tabw->showPage(i); }
